/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.ereigniskal.ereigniskal;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.SortedMap;

import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.Atomar;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.DatumJahr;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.DatumVonBis;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.DefinierterEintrag;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.LogischeVerknuepfung;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

// import sys.funclib.debug.Debug;

/**
 * Die Klasse parst SystemKalenderEintraege aller mglichen Typen
 * 
 * @version $Revision: 1.2 $ / $Date: 2010/07/30 08:55:48 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 * 
 */
public class Parser
{

  private SystemkalenderEintrag _ske;
  
  
  /**
   * Der Debugger, stellt verschiedene Protokollierungsfunktionen zur Verfgung
   */
  private Debug _debug;

  /**
   * Konstruktor der Klasse
   */
  public Parser()
  {
    _debug = Debug.getLogger();
  }
  

  /**
   * Parst die verschieden Typen von SystemKalenderEintraegen
   * 
   * @param pid
   *          Die Pid
   * @param definition
   *          Die definition des Ske
   * @return Boolean true, wenn ein gltiger Ske einem der definierten Typen entspricht
   * @throws IllegalArgumentException
   *           Wenn einer der uebergebenen Argumente einen null-Wert enthaelt.
   */
  public Boolean parseSystemkalenderEintrag(String pid, String name, String definition) throws Exception
  {

    if (pid == null || definition == null)
      throw new IllegalArgumentException("null value");

    pid = pid.toLowerCase();
    name = name.toLowerCase();
    definition = definition.toLowerCase();

    SimpleDateFormat format = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Calendar cal = new GregorianCalendar().getInstance();    
    
    Integer jahr = cal.get(Calendar.YEAR);
    
    String s1 = "01.01." + jahr + " 00:00:00,000";   
   
    //jahr += 1;
    
    //String s2 = "01.01." + jahr + " 00:00:00,000";
    String s2 = "31.12." + jahr + " 23:59:59,999";
    
    Date d1 = format.parse(s1);
    Date d2 = format.parse(s2);

    if (definition.contains("<") || definition.contains("({"))
    {

      // Eintrag erzeugen und zur Liste der Systemkalendereintraege
      // hinzufuegen
      _ske = new DatumVonBis(pid, definition);
      _ske.setName(name);

      // Es wird geprueft ob ein neuer SystemKalenderEintrag gesendet wurde
      if (!EreignisKalender.getSkeList().containsKey(_ske.getPid()))
      {

        if (_ske.pruefeEintrag())
        {
          //_ske.berechneZustandsWechselZustand(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
          SortedMap<Long, Boolean> sm = _ske.berechneZustandsWechsel(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
          if (sm!= null)
            _ske.getListeZustandsWechsel().putAll(sm);
          
          
          EreignisKalender.getSkeList().put(pid, _ske);
                 
          return true;
        }

      }
      else
      {
        // SystemKalenderEintrag schon vorhanden
        //DatumVonBis tmp = (DatumVonBis)EreignisKalender.getSkeList().get(_ske.getPid());
        SystemkalenderEintrag tmp = EreignisKalender.getSkeList().get(_ske.getPid());
        String alt = tmp.getDefinition();
        String neu = _ske.getDefinition();

        // Es wird geprueft ob eine neue Definition gesendet wurde
        if (alt.equals(neu))
        {
          _debug.fine("Definition ist noch die Alte");
          return true;
        }
        else
        {
          if (_ske.pruefeEintrag())
          {            
            EreignisKalender.getSkeList().remove(_ske.getPid());
            
//            _ske.berechneZustandsWechselZustand(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));           
            SortedMap<Long, Boolean> sm = _ske.berechneZustandsWechsel(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));           
            if (sm!= null)
              _ske.getListeZustandsWechsel().putAll(sm);
            
            EreignisKalender.getSkeList().put(_ske.getPid(), _ske);

            return true;
          }
        }
      }
    }
    else if (definition.contains(".") && definition.contains(","))
    {

      //DatumJahr dtj = new DatumJahr(pid, definition);
      _ske = new DatumJahr(pid, definition);
      _ske.setName(name);

      // Es wird geprueft ob ein neuer SystemKalenderEintrag gesendet wurde
      if (!EreignisKalender.getSkeList().containsKey(_ske.getPid()))
      {

        if (_ske.pruefeEintrag())
        {

//          _ske.berechneZustandsWechselZustand(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
          SortedMap<Long, Boolean> sm = _ske.berechneZustandsWechsel(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
          if (sm!= null)
            _ske.getListeZustandsWechsel().putAll(sm);
          
          EreignisKalender.getSkeList().put(pid, _ske);
          
          return true;
        }

      }
      else
      {
        // SystemKalenderEintrag schon vorhanden
        //DatumJahr tmp = (DatumJahr)EreignisKalender.getSkeList().get(_ske.getPid());
        SystemkalenderEintrag tmp = EreignisKalender.getSkeList().get(_ske.getPid());
        String alt = tmp.getDefinition();
        String neu = _ske.getDefinition();

        // Es wird geprueft ob eine neue Definition gesendet wurde
        if (alt.equals(neu))
        {
          _debug.fine("Definition ist noch die Alte");
          return true;

        }
        else
        {
          if (_ske.pruefeEintrag())
          {
            EreignisKalender.getSkeList().remove(_ske.getPid());

//            _ske.berechneZustandsWechselZustand(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
            SortedMap<Long, Boolean> sm = _ske.berechneZustandsWechsel(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
            if (sm!= null)
              _ske.getListeZustandsWechsel().putAll(sm);
            
            EreignisKalender.getSkeList().put(_ske.getPid(), _ske);

            return true;
          }
        }
      }

    }
    else if (definition.equalsIgnoreCase("montag") || definition.equalsIgnoreCase("dienstag")
        || definition.equalsIgnoreCase("mittwoch") || definition.equalsIgnoreCase("donnerstag")
        || definition.equalsIgnoreCase("freitag") || definition.equalsIgnoreCase("samstag")
        || definition.equalsIgnoreCase("sonntag") || definition.equalsIgnoreCase("ostersonntag")
        || definition.equalsIgnoreCase("tag"))
    {

      // Eintrag erzeugen und zur Liste der Systemkalendereintraege
      // hinzufuegen
      _ske = new Atomar(pid, definition);
      _ske.setName(name);
      
//      SortedMap<Long, Boolean> l = _ske.berechneZustandsWechselZustand(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
      SortedMap<Long, Boolean> sm = _ske.berechneZustandsWechsel(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
      if (sm!= null)
        _ske.getListeZustandsWechsel().putAll(sm);
      
     

      EreignisKalender.getSkeList().put(pid, _ske);
     

      return true;

    }
    else if ((definition.contains("und") || definition.contains("oder") || definition.contains("nicht")))
    {

      // Eintrag erzeugen und zur Liste der Systemkalendereintraege
      // hinzufuegen
      //LogischeVerknuepfung lvk = new LogischeVerknuepfung(EreignisKalender.getSkeList(), pid, definition);
      _ske = new LogischeVerknuepfung(EreignisKalender.getSkeList(), pid, definition);
      _ske.setName(name);

      // Es wird geprueft ob ein neuer SystemKalenderEintrag gesendet wurde
      if (!EreignisKalender.getSkeList().containsKey(_ske.getPid()))
      {

        if (_ske.pruefeEintrag())
        {         
//          _ske.berechneZustandsWechselZustand(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
          SortedMap<Long, Boolean> sm = _ske.berechneZustandsWechsel(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
          if (sm!= null)
            _ske.getListeZustandsWechsel().putAll(sm);
          
          EreignisKalender.getSkeList().put(pid, _ske);
         
          return true;
        }

      }
      else
      {
        // SystemKalenderEintrag schon vorhanden
        //LogischeVerknuepfung tmp = (LogischeVerknuepfung)EreignisKalender.getSkeList().get(_ske.getPid());
        SystemkalenderEintrag tmp = EreignisKalender.getSkeList().get(_ske.getPid());
        String alt = tmp.getDefinition();
        String neu = _ske.getDefinition();

        // Es wird geprueft ob eine neue Definition gesendet wurde
        if (alt.equals(neu))
        {
          _debug.fine("Definition ist noch die Alte");
          return true;
        }
        else
        {
          if (_ske.pruefeEintrag())
          {
            
            EreignisKalender.getSkeList().remove(_ske.getPid());
            
//            _ske.berechneZustandsWechselZustand(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
            SortedMap<Long, Boolean> sm = _ske.berechneZustandsWechsel(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
            if (sm!= null)
              _ske.getListeZustandsWechsel().putAll(sm);
            
            EreignisKalender.getSkeList().put(_ske.getPid(), _ske);
            return true;
          }
        }
      }

    }
    else if ((definition.contains("+") || definition.contains("-")))
    {
      _ske = new DefinierterEintrag(EreignisKalender.getSkeList(), pid, definition);
      _ske.setName(name);

      // Es wird geprueft ob ein neuer SystemKalenderEintrag gesendet wurde
      if (!EreignisKalender.getSkeList().containsKey(_ske.getPid()))
      {

        if (_ske.pruefeEintrag())
        {
         
//          _ske.berechneZustandsWechselZustand(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
          SortedMap<Long, Boolean> sm = _ske.berechneZustandsWechsel(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
          if (sm!= null)
            _ske.getListeZustandsWechsel().putAll(sm);
          
          EreignisKalender.getSkeList().put(pid, _ske);
          
          return true;

        }

      }
      else
      {
        // SystemKalenderEintrag schon vorhanden
        //DatumVonBis tmp = (DatumVonBis)EreignisKalender.getSkeList().get(_ske.getPid());
        SystemkalenderEintrag tmp = EreignisKalender.getSkeList().get(_ske.getPid());
        String alt = tmp.getDefinition();
        String neu = _ske.getDefinition();

        // Es wird geprueft ob eine neue Definition gesendet wurde
        if (alt.equals(neu))
        {
          _debug.fine("Definition ist noch die Alte");
          return true;
        }
        else
        {
          if (_ske.pruefeEintrag())
          {
            EreignisKalender.getSkeList().remove(_ske.getPid());
            
//            _ske.berechneZustandsWechselZustand(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
            SortedMap<Long, Boolean> sm = _ske.berechneZustandsWechsel(d1.getTime(), d2.getTime(), cal.get(Calendar.YEAR));
            if (sm!= null)
              _ske.getListeZustandsWechsel().putAll(sm);
            
            EreignisKalender.getSkeList().put(_ske.getPid(), _ske);

            return true;
          }
        }
      }

    }else{
           
      Map<String, SystemkalenderEintrag> skeList = EreignisKalender.getSkeList();
      
      if (skeList.containsKey("ske." + definition)){
        
        
        SystemkalenderEintrag ske = skeList.get("ske." + definition);
        
        if (ske instanceof DatumJahr)
        {
          _ske = (DatumJahr)ske;                    
          
        }
        else if (ske instanceof DatumVonBis)
        {
          _ske = (DatumVonBis)ske;
          
        }
        else if (ske instanceof LogischeVerknuepfung)
        {
          _ske = (LogischeVerknuepfung)ske;  
        }
        else if (ske instanceof DefinierterEintrag)
        {
          _ske = (DefinierterEintrag)ske; 
        }else{          
          //return false;
          
        }
        
        EreignisKalender.getSkeList().put(pid, _ske);
        return true;
        
      }
    }

    // Systemkalendereintrag konnte nicht geparst werden!
    return false;

  }

}
